<?php

require_once 'clients/base/api/ConfigModuleApi.php';
require_once 'modules/PX_NeedToDoEvents/PX_NeedToDoEvents.php';

class PX_NeedToDoEventsConfigApi extends ConfigModuleApi
{

    /**
     * @return array
     */
    public function registerApiRest() {
        return array(
            'configRead' => array(
                'reqType' => 'GET',
                'path' => array('PX_NeedToDoEvents', 'config'),
                'pathVars' => array('module'),
                'method' => 'getConfig',
                'shortHelp' => 'Retrieves the config settings for a given module',
                'longHelp' => 'include/api/help/module_config_get_help.html',
            ),
            'configCreate' => array(
                'reqType' => 'POST',
                'path' => array('PX_NeedToDoEvents', 'config'),
                'pathVars' => array('module', ''),
                'method' => 'configSave',
                'shortHelp' => 'Creates the config entries for the given module',
                'longHelp' => 'include/api/help/module_config_post_help.html',
            ),
            'configUpdate' => array(
                'reqType' => 'PUT',
                'path' => array('PX_NeedToDoEvents', 'config'),
                'pathVars' => array('module', ''),
                'method' => 'configSave',
                'shortHelp' => 'Updates the config entries for given module',
                'longHelp' => 'include/api/help/module_config_put_help.html',
            ),
            'grantUserAccess' => array(
                'reqType' => 'PUT',
                'path' => array('PX_NeedToDoEvents', 'license', 'grant-user-access'),
                'pathVars' => array('module'),
                'method' => 'grantUserAccess',
                'shortHelp' => 'Grants users access to Need To Do Viewer Plugin',
                'longHelp' => '',
            ),
            'revokeUserAccess' => array(
                'reqType' => 'PUT',
                'path' => array('PX_NeedToDoEvents', 'license', 'revoke-user-access'),
                'pathVars' => array('module'),
                'method' => 'revokeUserAccess',
                'shortHelp' => 'Revokes users access to Need To Do Viewer Plugin',
                'longHelp' => '',
            ),
            'validateLicense' => array(
                'reqType' => 'GET',
                'path' => array('PX_NeedToDoEvents', 'license', 'validate'),
                'pathVars' => array('', '', ''),
                'method' => 'validateLicense',
                'shortHelp' => 'Validate user access',
                'longHelp' => '',
            ),
        );
    }

    /**
     * Returns the config settings for the given module
     *
     * @param ServiceBase $api
     * @param array $args 'module' is required, 'platform' is optional and defaults to 'base'
     * @return array
     */
    public function getConfig(ServiceBase $api, array $args)
    {
        $package = new \Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\Package\Package();
        $licenseType = $package->getPackageLicenseType();
        if($licenseType == \Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\Package\Package::SUGAR_OUTFITTERS_LICENSE_TYPE){
            return parent::config($api, $args);
        }

        $control = new \Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\Package\License\Control();
        $panel = new \Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\Package\Package();

        $config = array(
            'license_token' => $control->getLicenseToken(),
            'valid' => true,
            'expiry_date' => null,
            'users_limit' => null,
            'active_users' => array(),
            'license_type' => $control->getLicenseType(),

            'version' => $panel->getPackageVersion(),
        );

        try{
            $expireDate = $control->getExpireDate();
            $config['expiry_date'] = !empty($expireDate) ? $expireDate->format('Y-m-d') : null;
        }catch(Exception $e){
        }

        try{
            $config['users_limit'] = $control->getUserLimit();
        }catch(Exception $e){
        }

        try{
            $config['active_users'] = $control->getCurrentUsers();
        }catch(Exception $e){
        }

        try {
            $control->checkLicense();
        } catch (\Exception $e) {
            $config['valid'] = false;
        }

        return $config;
    }

    /**
     * Save function for the config settings for PX_NeedToDoEvents module.
     *
     * @param ServiceBase $api
     * @param array $args 'module' is required, 'platform' is optional and defaults to 'base'
     * @return array
     * @throws SugarApiExceptionMissingParameter
     * @throws SugarApiExceptionNotAuthorized
     */
    public function configSave(ServiceBase $api, array $args)
    {

        $package = new \Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\Package\Package();
        $licenseType = $package->getPackageLicenseType();
        if($licenseType == \Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\Package\Package::SUGAR_OUTFITTERS_LICENSE_TYPE){
            return parent::configSave($api, $args);
        }

        $this->requireArgs($args, array('license_token'));
        $this->skipMetadataRefresh = true;
        parent::configSave($api, $args);

        $panel = new \Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\Package\Package();
        $panel->validateLicense($args['license_token']);

        return $this->getConfig($api, $args);
    }

    /**
     * Grant user access
     *
     * @param ServiceBase $api
     * @param array $args 'ids' required
     * @return array
     * @throws SugarApiExceptionInvalidParameter
     * @throws SugarApiExceptionMissingParameter
     */
    public function grantUserAccess(ServiceBase $api, array $args)
    {
        $this->requireArgs($args, array('ids'));

        $result = array(
            'status' => 'success',
            'addedUsers' => 0,
            'activeUsers' => 0,
            'config' => array()
        );

        $control = new \Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\Package\License\Control();
        $maximumLimit = $control->getUserLimit();
        $currentAmount = $control->getCurrentUsers();
        $addedUsers = 0;
        $throwException = false;

        if($currentAmount < $maximumLimit) {
            $userIds = $args['ids'];

            foreach($userIds as $userId){
                $userBean = BeanFactory::retrieveBean('Users', $userId);
                if($userBean && !$userBean->px_ntdv_access_c){
                    if(($currentAmount + $addedUsers) >= $maximumLimit) {
                        $throwException = true;
                        break;
                    }

                    $addedUsers++;
                    $userBean->px_ntdv_access_c = 1;
                    $userBean->save();
                }
            }

        }else{
            $throwException = true;
        }

        if($throwException){
            $result['status'] = 'failure';
            $errorLabel = translate('ERROR_LICENSE_MAX_USERS_LIMIT_EXCEEDED', 'PX_NeedToDoEvents');
            if(!empty($args['throw_errors']) && $args['throw_errors'])
                throw new \SugarApiExceptionInvalidParameter($errorLabel);
            else
                $result['errorLabel'] = $errorLabel;
        }

        $result['addedUsers'] = $addedUsers;
        $result['activeUsers'] = $currentAmount + $addedUsers;

        \Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\Package\License\Control::clearCachedData();
        $result['config'] = $this->getConfig($api, $args);

        return $result;
    }

    /**
     * Revoke user access
     *
     * @param ServiceBase $api
     * @param array $args 'ids' is required
     * @return array
     * @throws SugarApiExceptionMissingParameter
     */
    public function revokeUserAccess(ServiceBase $api, array $args)
    {
        $this->requireArgs($args, array('ids'));

        $result = array(
            'status' => 'success',
            'config' => array()
        );

        $control = new \Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\Package\License\Control();
        $currentAmount = $control->getCurrentUsers();
        $removedUsers = 0;

        foreach ($args['ids'] as $id) {
            $userBean = BeanFactory::retrieveBean('Users', $id);

            if ($userBean && $userBean->px_ntdv_access_c) {
                $removedUsers++;
                $userBean->px_ntdv_access_c = 0;

                $userBean->save();
            }
        }

        $result['removed_users'] = $removedUsers;
        $result['activeUsers'] = $currentAmount - $removedUsers;

        \Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\Package\License\Control::clearCachedData();
        $result['config'] = $this->getConfig($api, $args);

        return $result;
    }

    public function validateLicense(ServiceBase $api, array $args)
    {
        $control = new \Sugarcrm\Sugarcrm\custom\PxRoutines\NeedToDoViewer\Package\License\Control();
        $control->checkLicenseForCurrentUser();
        return array ();
    }

}
